package com.agilefaqs.rft.suggestions;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.reset;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.agilefaqs.rft.domain.DomainNameService;
import com.agilefaqs.rft.suggestions.IDTokens;
import com.agilefaqs.rft.suggestions.SuggestionsUtil;

public class SuggestionsUtilTest {
    private final String token1 = "first_name";
    private final String token2 = "last_name";
    private final DomainNameService domainNameService = createMock(DomainNameService.class);
    private final SuggestionsUtil utils = new SuggestionsUtil(domainNameService);

    @Before
    public void setUp() {
        reset(domainNameService);

    }

    public void stopRecordingAndStartReplay() {
        replay(domainNameService);

    }

    @After
    public void verifyMocks() {
        verify(domainNameService);

    }

    @Test
    public void shouldGenerateValidIdentityTokensBasedOnPassedTokensIfBothAreNotRestricted() throws Exception {
        expect(domainNameService.isCelebrityName(token1, token2)).andReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(token1)).andReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(token2)).andReturn(null);
        expect(domainNameService.isCelebrityName(token1, token2)).andReturn(false);
        stopRecordingAndStartReplay();
        IDTokens IdTokens = utils.getIdentityTokens(token1, token2);
        final String generatedToken1 = IdTokens.getToken1();
        final String generatedToken2 = IdTokens.getToken2();
        assertEquals("expected was " + token1 + " but was : " + generatedToken1, generatedToken1, token1);
        assertEquals("expected was " + token2 + " but was : " + generatedToken2, generatedToken2, token2);
    }

    @Test
    public void shouldGenerateValidIdentityTokensWhenTheFirstTokenAreRestricted() throws Exception {
        final String expectedToken1 = "first_nam";
        expect(domainNameService.isCelebrityName(token1, token2)).andReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(token1)).andReturn(token1);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(expectedToken1)).andReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(token2)).andReturn(null);
        expect(domainNameService.isCelebrityName(expectedToken1, token2)).andReturn(false);
        stopRecordingAndStartReplay();
        IDTokens IdTokens = utils.getIdentityTokens(token1, token2);
        final String generatedToken1 = IdTokens.getToken1();
        String generatedToken2 = IdTokens.getToken2();
        assertEquals("expected was " + expectedToken1 + " but was : " + generatedToken1, generatedToken1, expectedToken1);
        assertEquals("expected was " + token2 + " but was : " + generatedToken2, generatedToken2, token2);
    }

    @Test
    public void shouldGenerateValidIdentityTokensWhenTheFirstTokenIsRestrictedAndLoopThrice() throws Exception {
        final String attempt1Token = "first_nam";
        final String attempt2Token = "first_na";
        final String attempt3Token = "first_n";
        expect(domainNameService.isCelebrityName(token1, token2)).andReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(token1)).andReturn(token1);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt1Token)).andReturn(attempt1Token);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt2Token)).andReturn(attempt2Token);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt3Token)).andReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(token2)).andReturn(null);
        expect(domainNameService.isCelebrityName(attempt3Token, token2)).andReturn(false);
        stopRecordingAndStartReplay();
        IDTokens IdTokens = utils.getIdentityTokens(token1, token2);
        final String generatedToken1 = IdTokens.getToken1();
        String generatedToken2 = IdTokens.getToken2();
        assertEquals("expected was " + attempt3Token + " but was : " + generatedToken1, generatedToken1, attempt3Token);
        assertEquals("expected was " + token2 + " but was : " + generatedToken2, generatedToken2, token2);
    }

    @Test
    public void shouldGenerateValidIdentityTokensWhenTheSecondTokenAreRestricted() throws Exception {
        final String expectedToken2 = "last_nam";
        expect(domainNameService.isCelebrityName(token1, token2)).andReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(token1)).andReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(token2)).andReturn(token2);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(expectedToken2)).andReturn(null);
        expect(domainNameService.isCelebrityName(token1, expectedToken2)).andReturn(false);
        stopRecordingAndStartReplay();
        IDTokens IdTokens = utils.getIdentityTokens(token1, token2);
        final String generatedToken1 = IdTokens.getToken1();
        String generatedToken2 = IdTokens.getToken2();
        assertEquals("expected was " + token1 + " but was : " + generatedToken1, generatedToken1, token1);
        assertEquals("expected was " + expectedToken2 + " but was : " + generatedToken2, generatedToken2, expectedToken2);
    }

    @Test
    public void shouldGenerateValidIdentityTokensWhenTheSecondTokenIsRestrictedAndLoopThrice() throws Exception {
        final String attempt1Token = "last_nam";
        final String attempt2Token = "last_na";
        final String attempt3Token = "last_n";
        expect(domainNameService.isCelebrityName(token1, token2)).andReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(token1)).andReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(token2)).andReturn(token2);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt1Token)).andReturn(attempt1Token);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt2Token)).andReturn(attempt2Token);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt3Token)).andReturn(null);
        expect(domainNameService.isCelebrityName(token1, attempt3Token)).andReturn(false);
        stopRecordingAndStartReplay();
        IDTokens IdTokens = utils.getIdentityTokens(token1, token2);
        final String generatedToken1 = IdTokens.getToken1();
        String generatedToken2 = IdTokens.getToken2();
        assertEquals("expected was " + token1 + " but was : " + generatedToken1, generatedToken1, token1);
        assertEquals("expected was " + attempt3Token + " but was : " + generatedToken2, generatedToken2, attempt3Token);
    }

    @Test
    public void shouldGenerateValidIdentityTokensBasedOnPassedTokensIfBothAreRestricted() throws Exception {
        final String attempt1Token1 = "first_nam";
        final String attempt2Token1 = "first_na";
        final String attempt3Token1 = "first_n";
        final String attempt1Token2 = "last_nam";
        final String attempt2Token2 = "last_na";
        final String attempt3Token2 = "last_n";
        expect(domainNameService.isCelebrityName(token1, token2)).andReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(token1)).andReturn(token1);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt1Token1)).andReturn(attempt1Token1);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt2Token1)).andReturn(attempt2Token1);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt3Token1)).andReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(token2)).andReturn(token2);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt1Token2)).andReturn(attempt1Token2);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt2Token2)).andReturn(attempt2Token2);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt3Token2)).andReturn(null);
        expect(domainNameService.isCelebrityName(attempt3Token1, attempt3Token2)).andReturn(false);
        stopRecordingAndStartReplay();
        IDTokens IdTokens = utils.getIdentityTokens(token1, token2);
        final String generatedToken1 = IdTokens.getToken1();
        String generatedToken2 = IdTokens.getToken2();
        assertEquals("expected was " + attempt3Token1 + " but was : " + generatedToken1, generatedToken1, attempt3Token1);
        assertEquals("expected was " + attempt3Token2 + " but was : " + generatedToken2, generatedToken2, attempt3Token2);
    }

    @Test
    public void generateIdTokensShouldReturnNullIfMaxNumberOfAttemptsAreMadeForFirstToken() throws Exception {
        final String attempt1Token = "first_nam";
        final String attempt2Token = "first_na";
        final String attempt3Token = "first_n";
        final String attempt4Token = "first_";
        expect(domainNameService.isCelebrityName(token1, token2)).andReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(token1)).andReturn(token1);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt1Token)).andReturn(attempt1Token);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt2Token)).andReturn(attempt2Token);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt3Token)).andReturn(attempt3Token);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt4Token)).andReturn(attempt4Token);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(token2)).andReturn(null);
        stopRecordingAndStartReplay();
        IDTokens IdTokens = utils.getIdentityTokens(token1, token2);
        assertNull(IdTokens);

    }

    @Test
    public void generateIdTokensShouldReturnNullIfMaxNumberOfAttemptsAreMadeForSecondToken() throws Exception {
        final String attempt1Token = "last_nam";
        final String attempt2Token = "last_na";
        final String attempt3Token = "last_n";
        final String attempt4Token = "last_";
        expect(domainNameService.isCelebrityName(token1, token2)).andReturn(false);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(token1)).andReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(token2)).andReturn(token2);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt1Token)).andReturn(attempt1Token);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt2Token)).andReturn(attempt2Token);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt3Token)).andReturn(attempt3Token);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt4Token)).andReturn(attempt4Token);
        stopRecordingAndStartReplay();
        IDTokens IdTokens = utils.getIdentityTokens(token1, token2);
        assertNull(IdTokens);

    }

    @Test
    public void generateIdTokensShouldReturnASuggestionIfCombinationOFTokensIsACeleBrity() throws Exception {
        final String attempt1Token1 = "first_nam";
        final String attempt1Token2 = "last_nam";
        expect(domainNameService.isCelebrityName(token1, token2)).andReturn(true);
        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt1Token1)).andReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt1Token2)).andReturn(null);
        expect(domainNameService.isCelebrityName(attempt1Token1, attempt1Token2)).andReturn(false);
        stopRecordingAndStartReplay();

        IDTokens IdTokens = utils.getIdentityTokens(token1, token2);
        final String generatedToken1 = IdTokens.getToken1();
        String generatedToken2 = IdTokens.getToken2();
        assertEquals("expected was " + attempt1Token1 + " but was : " + generatedToken1, generatedToken1, attempt1Token1);
        assertEquals("expected was " + attempt1Token2 + " but was : " + generatedToken2, generatedToken2, attempt1Token2);

    }

    @Test
    public void generateIdTokensShouldReturnNullIfCombinationOFTokensIsACeleBrityAndFiveAttempts() throws Exception {
        final String attempt1Token1 = "first_nam";
        final String attempt2Token1 = "first_na";
        final String attempt3Token1 = "first_n";
        final String attempt4Token1 = "first_";
        final String attempt1Token2 = "last_nam";
        final String attempt2Token2 = "last_na";
        final String attempt3Token2 = "last_n";
        final String attempt4Token2 = "last_";
        expect(domainNameService.isCelebrityName(token1, token2)).andReturn(true);

        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt1Token1)).andReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt1Token2)).andReturn(null);
        expect(domainNameService.isCelebrityName(attempt1Token1, attempt1Token2)).andReturn(true);

        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt2Token1)).andReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt2Token2)).andReturn(null);
        expect(domainNameService.isCelebrityName(attempt2Token1, attempt2Token2)).andReturn(true);

        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt3Token1)).andReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt3Token2)).andReturn(null);
        expect(domainNameService.isCelebrityName(attempt3Token1, attempt3Token2)).andReturn(true);

        expect(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt4Token1)).andReturn(null);
        expect(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt4Token2)).andReturn(null);
        expect(domainNameService.isCelebrityName(attempt4Token1, attempt4Token2)).andReturn(true);
        stopRecordingAndStartReplay();

        IDTokens IdTokens = utils.getIdentityTokens(token1, token2);
        assertNull(IdTokens);

    }

}
